/*
 * Copyright (c) 2007-2010 Sonatype, Inc.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package org.sonatype.sisu.assembler.gshell.internal;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.inject.Inject;
import javax.inject.Singleton;

import org.codehaus.plexus.util.DirectoryScanner;
import org.sonatype.eclipse.bridge.EclipseBridge;
import org.sonatype.eclipse.bridge.EclipseInstance;
import org.sonatype.eclipse.bridge.EclipseLocation;
import org.sonatype.eclipse.bridge.EclipseLocationFactory;
import org.sonatype.gshell.file.FileSystemAccess;
import org.sonatype.gshell.plexus.PlexusRuntime;

@Singleton
public class PreparedEclipseInstance
    implements EclipseInstance
{

    private EclipseInstance wrapped;

    private final PlexusRuntime plexusRuntime;

    private final FileSystemAccess fileSystem;

    @Inject
    public PreparedEclipseInstance( final PlexusRuntime plexusRuntime, final FileSystemAccess fileSystem )
    {
        assert plexusRuntime != null;
        assert fileSystem != null;

        this.plexusRuntime = plexusRuntime;
        this.fileSystem = fileSystem;
    }

    public <T> T getService( final Class<T> serviceType )
    {
        initialize();
        return wrapped.getService( serviceType );
    }

    public Long installBundle( final String location )
    {
        initialize();
        return wrapped.installBundle( location );
    }

    public Long installBundle( final String location, final InputStream inputStream )
    {
        initialize();
        return wrapped.installBundle( location, inputStream );
    }

    public EclipseInstance shutdown()
    {
        initialize();
        return wrapped.shutdown();
    }

    public EclipseInstance start( final Map<String, String> launchProperties )
    {
        initialize();
        return wrapped.start( launchProperties );
    }

    public void startBundle( final Long id )
    {
        initialize();
        wrapped.startBundle( id );
    }

    private void initialize()
    {
        if ( wrapped != null )
        {
            return;
        }

        try
        {
            final EclipseBridge bridge = plexusRuntime.lookup( EclipseBridge.class );
            final EclipseLocationFactory locationFactory = plexusRuntime.lookup( EclipseLocationFactory.class );
            final EclipseLocation location =
                locationFactory.createPackedEclipseLocation( getEclipseArchive( new File( fileSystem.getShellHomeDir(),
                    "p2-bridge/runtime" ) ) );
            wrapped = bridge.createInstance( location );

            start( getLaunchProperties() );

            for ( final File bundle : getBundles( new File( fileSystem.getShellHomeDir(), "p2-bridge/bundles" ) ) )
            {
                startBundle( installBundle( bundle.getCanonicalFile().toURI().toASCIIString() ) );
            }
        }
        catch ( final Exception e )
        {
            // TODO shall we throw this?
            throw new RuntimeException( e );
        }
    }

    private Map<String, String> getLaunchProperties()
    {
        final Map<String, String> launchProperties = new HashMap<String, String>();
        launchProperties.putAll( EclipseInstance.DEFAULT_LAUNCH_PROPERTIES );

        // Better implementation that looks up the exported packages from the artifact
        launchProperties.put(
            "org.osgi.framework.system.packages.extra",
            "org.sonatype.p2.bridge;version=\"1.0.0\",org.sonatype.p2.bridge.model;version=\"1.0.0\"" );

        return launchProperties;
    }

    private static File getEclipseArchive( final File runtimeDir )
        throws IOException
    {
        final DirectoryScanner ds = new DirectoryScanner();
        ds.setBasedir( runtimeDir );
        ds.setIncludes( new String[] { "*.zip" } );
        ds.scan();
        final String[] files = ds.getIncludedFiles();
        if ( files == null || files.length == 0 )
        {
            throw new RuntimeException( String.format( "Could not find p2 runtime in %s", runtimeDir.getAbsolutePath() ) );
        }
        else if ( files.length > 1 )
        {
            throw new RuntimeException( String.format( "More then one p2 runtime found in %s",
                runtimeDir.getAbsolutePath() ) );
        }
        return new File( runtimeDir, files[0] ).getCanonicalFile();
    }

    private static Collection<File> getBundles( final File bundlesDir )
        throws IOException
    {
        final DirectoryScanner ds = new DirectoryScanner();
        ds.setBasedir( bundlesDir );
        ds.setIncludes( new String[] { "**/*.jar" } );
        ds.scan();

        final Collection<File> files = new ArrayList<File>();
        final String[] scanned = ds.getIncludedFiles();
        for ( final String name : scanned )
        {
            files.add( new File( bundlesDir, name ) );
        }
        return files;
    }
}
