/*
 * Copyright (c) 2007-2010 Sonatype, Inc.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package org.sonatype.sisu.assembler.gshell.internal;

import static org.sonatype.sisu.assembler.gshell.internal.PrintUtils.printHeader;

import java.io.File;
import java.util.List;

import org.sonatype.gshell.command.Command;
import org.sonatype.gshell.command.CommandContext;
import org.sonatype.gshell.command.IO;
import org.sonatype.gshell.file.FileSystemAccess;
import org.sonatype.gshell.util.cli2.Argument;
import org.sonatype.gshell.util.cli2.Option;
import org.sonatype.p2.bridge.IUIdentity;
import org.sonatype.p2.bridge.P2Director;
import org.sonatype.p2.bridge.P2ProfileRegistry;
import org.sonatype.p2.bridge.ProfileTimestamp;

import com.google.inject.Inject;

@Command( name = "p2/rollback" )
public class RollbackCommand
    extends P2Command
{

    @Argument( required = true )
    private String timestamp;

    @Option( name = "p", longName = "profile" )
    private final String profile = "master";

    @Option( name = "t", longName = "tag" )
    private String tag;

    private final FileSystemAccess fileSystem;

    private final P2Director p2Director;

    private final P2ProfileRegistry p2ProfileRegistry;

    private final RepositoryRegistry repositoryRegistry;

    private final List<ProfileTimestamp> latestListed;

    @Inject
    public RollbackCommand( final FileSystemAccess fileSystem, final P2ProfileRegistry p2ProfileRegistry,
                            final P2Director p2Director, final RepositoryRegistry repositoryRegistry,
                            @ListHistoryCommand.Latest final List<ProfileTimestamp> latestListed )
    {
        assert fileSystem != null;
        assert p2ProfileRegistry != null;
        assert p2Director != null;
        assert repositoryRegistry != null;
        assert latestListed != null;

        this.fileSystem = fileSystem;
        this.p2ProfileRegistry = p2ProfileRegistry;
        this.p2Director = p2Director;
        this.repositoryRegistry = repositoryRegistry;
        this.latestListed = latestListed;
    }

    public Object execute( final CommandContext context )
        throws Exception
    {
        assert context != null;

        final IO io = context.getIo();

        if ( repositoryRegistry.isEmpty() )
        {
            io.error( "There should be at least one repository configured to be able to rollback." );
            return Result.FAILURE;
        }

        final File location = getLocation( fileSystem );

        final IOLogProxy logProxy = new IOLogProxy( io );

        Long actualTimestamp = null;

        try
        {
            final Integer idx = Integer.valueOf( timestamp );
            final ProfileTimestamp listed = latestListed.get( idx - 1 );
            actualTimestamp = listed.getTimestamp();
        }
        catch ( final Exception e )
        {
            // do nothing, value is not an index position
        }

        if ( actualTimestamp == null )
        {
            final ProfileTimestamp[] timestamps = p2ProfileRegistry.getProfileTimestamps( location, profile );
            if ( timestamps != null )
            {
                for ( final ProfileTimestamp profileTimestamp : timestamps )
                {
                    if ( timestamp.equals( profileTimestamp.getTag() ) )
                    {
                        actualTimestamp = profileTimestamp.getTimestamp();
                    }
                    else
                    {
                        for ( final IUIdentity iu : profileTimestamp.getRoots() )
                        {
                            if ( timestamp.equals( iu.toString() ) || timestamp.equals( iu.getVersion() ) )
                            {
                                actualTimestamp = profileTimestamp.getTimestamp();
                            }
                        }
                    }
                }
            }
            if ( actualTimestamp == null )
            {
                try
                {
                    actualTimestamp = Long.valueOf( timestamp );
                }
                catch ( final NumberFormatException ignore )
                {
                    throw new RuntimeException( String.format( "Rollback timestamp [%s] is not a valid tag", timestamp ) );
                }
            }
        }

        printHeader( io, String.format( "Rollback to %s:", actualTimestamp ) );

        if ( tag == null )
        {
            tag = String.format( "Rollback to %s", actualTimestamp );
        }

        p2Director.rollback( logProxy, location, profile, actualTimestamp, repositoryRegistry.get(), tag );

        io.println( "" );

        return Result.SUCCESS;
    }

}
